@@ -7,10 +7,12 @@ create_test_sleeplog_csv = function(Nnights = 7,
   # Nnights is the number of nights the sleeplog has
   if (length(storagelocation) == 0) storagelocation = getwd()
   if (Nnights == 0) Nnights = 1
-  Nnights = 7
   if (advanced == FALSE) {
     sleeplog = as.data.frame(t(c("123",rep(c("23:00:00","07:00:00"),Nnights))), stringsAsFactors = TRUE)
-    sleeplog[, 4:5] = c("03:00:00", "17:00:00") # let's add one night where the person slept untill the afternoon
+    if (ncol(sleeplog) >= 5) {
+      # let's add one night where the person slept untill the afternoon
+      sleeplog[, 4:5] = c("03:00:00", "17:00:00") 
+    }
     colnames(sleeplog) = c("ID",rep(c("onset","wakeup"),Nnights))
     write.table(
       sleeplog,
@@ -22,8 +24,7 @@ create_test_sleeplog_csv = function(Nnights = 7,
     )
   } else {
     times = c("07:00:00", "23:00:00", "13:00:00", "13:30:00", "15:00:00", "15:40:00",  "17:00:00", "17:15:00")
-    # when creating calendar dates create one date extra compared with number of nights
-    dates = seq(as.Date("2016/06/25"), as.Date("2016/06/25") + Nnights + 1, by = 1)
+    dates = seq(as.Date("2016/06/25"), as.Date("2016/06/25") + Nnights, by = 1)
     Cnames = c(
       "_wakeup",
       "_inbed",
@@ -37,7 +38,8 @@ create_test_sleeplog_csv = function(Nnights = 7,
     names = "ID"
     log = c()
     for (di in 1:length(dates)) {
-      if (di < 10) { # include day number in the time, to ease testing sleeplog processing
+      # include day number in the time, to ease testing sleeplog processing
+      if (di < 10) {
         times2 = gsub(pattern = ":00:00", replacement = paste0(":00:0",di), x = times)
       } else {
         times2 = gsub(pattern = ":00:00", replacement = paste0(":00:", ifelse(di < 59, yes = di, no = "00")),
