@@ -219,7 +219,9 @@ g.getmeta = function(datafile, params_metrics = c(), params_rawdata = c(),
         }
       }
 
-      if (params_rawdata[["imputeTimegaps"]] && (dformat == FORMAT$CSV || dformat == FORMAT$GT3X)) {
+      if (params_rawdata[["imputeTimegaps"]] && (dformat == FORMAT$CSV ||
+                                                 dformat == FORMAT$AD_HOC_CSV ||
+                                                 dformat == FORMAT$GT3X)) {
         P = g.imputeTimegaps(data, sf = sf, k = 0.25,
                              PreviousLastValue = PreviousLastValue,
                              PreviousLastTime = PreviousLastTime,
@@ -227,7 +229,7 @@ g.getmeta = function(datafile, params_metrics = c(), params_rawdata = c(),
         data = P$x
         PreviousLastValue = data[nrow(data), c("x", "y", "z")]
         if ("time" %in% colnames(data)) {
-          PreviousLastTime = as.POSIXct(data$time[nrow(data)])
+          PreviousLastTime = as.POSIXct(data$time[nrow(data)], origin = "1970-1-1")
         } else {
           PreviousLastTime = NULL
         }
